@@ -1,46 +1,73 @@
 
-## This is how the RDS file is created:
-
-'
-json <- "https://raw.githubusercontent.com/sindresorhus/cli-spinners/dac4fc6571059bb9e9bc204711e9dfe8f72e5c6f/spinners.json"
-parsed <- jsonlite::fromJSON(json, simplifyVector = TRUE)
-pasis <- lapply(parsed, function(x) { x$frames <- I(x$frames); x })
-pdt <- as.data.frame(do.call(rbind, pasis))
-pdt$name <- rownames(pdt)
-rownames(pdt) <- NULL
-spinners <- pdt[, c("name", "interval", "frames")]
-usethis::use_data(spinners, internal = TRUE)
-'
+## See tools/spinners.R for how the RDS file is created
 
 #' Character vector to put a spinner on the screen
 #'
 #' `cli` contains many different spinners, you choose one according to your
 #' taste.
 #'
-#' @param which The name of the chosen spinner. The default depends on
-#'   whether the platform supports Unicode.
+#' ```{asciicast get-spinner}
+#' #| asciicast_at = "all",
+#' #| asciicast_knitr_output = "svg",
+#' #| asciicast_cursor = FALSE
+#' options(cli.spinner = "hearts")
+#' fun <- function() {
+#'   cli_progress_bar("Spinning")
+#'   for (i in 1:100) {
+#'     Sys.sleep(4/100)
+#'     cli_progress_update()
+#'   }
+#' }
+#' fun()
+#' options(cli.spinner = NULL)
+#' ```
+#'
+#' @param which The name of the chosen spinner. If `NULL`, then the default
+#'   is used, which can be customized via the `cli.spinner_unicode`,
+#'   `cli.spinner_ascii` and `cli.spinner` options. (The latter applies to
+#'   both Unicode and ASCII displays. These options can be set to the name
+#'   of a built-in spinner, or to a list that has an entry called `frames`,
+#'   a character vector of frames.
 #' @return A list with entries: `name`, `interval`: the suggested update
 #'   interval in milliseconds and `frames`: the character vector of the
 #'   spinner's frames.
 #'
 #' @family spinners
 #' @export
-#' @examples
-#' get_spinner()
-#' get_spinner("shark")
 
 get_spinner <- function(which = NULL) {
-  assert_that(is.null(which) || is_string(which))
+  stopifnot(is.null(which) || is_string(which) || is.list(which))
 
   if (is.null(which)) {
-    which <- if (is_utf8_output()) "dots" else "line"
+    if (is_utf8_output()) {
+      which <-
+        getOption("cli.spinner_unicode") %||%
+        getOption("cli.spinner") %||%
+        "dots"
+    } else {
+      which <-
+        getOption("cli.spinner_ascii") %||%
+        getOption("cli.spinner") %||%
+        "line"
+    }
   }
 
-  row <- match(which, spinners$name)
-  list(
-    name = which,
-    interval = spinners$interval[[row]],
-    frames = spinners$frames[[row]])
+  if (is.character(which)) {
+    row <- match(which, spinners$name)
+    which <- list(
+      name = which,
+      interval = spinners$interval[[row]],
+      frames = spinners$frames[[row]])
+  }
+
+  if (!is.character(which$frames)) {
+    stop("Spinner frames must be a character vector")
+  }
+
+  which$name <- which$name %||% NA_character_
+  which$interval <- which$interval %||% 100L
+
+  which
 }
 
 #' List all available spinners
@@ -63,6 +90,10 @@ list_spinners <- function() {
 #'   spinner itself will be substituted for `{spin}`. See example below.
 #' @param stream The stream to use for the spinner. Typically this is
 #'   standard error, or maybe the standard output stream.
+#'   It can also be a string, one of `"auto"`, `"message"`, `"stdout"`,
+#'   `"stderr"`. `"auto"` will select `stdout()` if the session is
+#'   interactive and there are no sinks, otherwise it will select
+#'   `stderr()`.
 #' @param static What to do if the terminal does not support dynamic
 #'   displays:
 #'   * `"dots"`: show a dot for each `$spin()` call.
@@ -84,28 +115,48 @@ list_spinners <- function() {
 #' The spinner is automatically throttled to its ideal update frequency.
 #'
 #' @section Examples:
-#' ```
+#'
 #' ## Default spinner
+#'
+
+#' ```{asciicast make-spinner-default}
+#' #| asciicast_at = "all",
+#' #| asciicast_knitr_output = "svg",
+#' #| asciicast_cursor = FALSE,
+#' #| asciicast_end_wait = 0
 #' sp1 <- make_spinner()
 #' fun_with_spinner <- function() {
 #'   lapply(1:100, function(x) { sp1$spin(); Sys.sleep(0.05) })
 #'   sp1$finish()
 #' }
 #' ansi_with_hidden_cursor(fun_with_spinner())
+#' ```
 #'
 #' ## Spinner with a template
+#'
+#' ```{asciicast make-spinner-template}
+#' #| asciicast_at = "all",
+#' #| asciicast_knitr_output = "svg",
+#' #| asciicast_cursor = FALSE,
+#' #| asciicast_end_wait = 0
 #' sp2 <- make_spinner(template = "Computing {spin}")
 #' fun_with_spinner2 <- function() {
 #'   lapply(1:100, function(x) { sp2$spin(); Sys.sleep(0.05) })
 #'   sp2$finish()
 #' }
 #' ansi_with_hidden_cursor(fun_with_spinner2())
+#' ```
 #'
 #' ## Custom spinner
+#'
+#' ```{asciicast make-spinner-custom}
+#' #| asciicast_at = "all",
+#' #| asciicast_knitr_output = "svg",
+#' #| asciicast_cursor = FALSE
 #' sp3 <- make_spinner("simpleDotsScrolling", template = "Downloading {spin}")
 #' fun_with_spinner3 <- function() {
 #'   lapply(1:100, function(x) { sp3$spin(); Sys.sleep(0.05) })
-#'   sp2$finish()
+#'   sp3$finish()
 #' }
 #' ansi_with_hidden_cursor(fun_with_spinner3())
 #' ```
@@ -113,22 +164,22 @@ list_spinners <- function() {
 #' @family spinners
 #' @export
 
-make_spinner <- function(which = NULL, stream = stderr(), template = "{spin}",
+make_spinner <- function(which = NULL, stream = "auto", template = "{spin}",
                          static = c("dots", "print", "print_line",
                                     "silent")) {
 
-  assert_that(
-    inherits(stream, "connection"),
+  stopifnot(
+    inherits(stream, "connection") || is_string(stream),
     is_string(template))
 
-  c_stream <- stream
+  c_stream <- get_real_output(stream)
   c_spinner <- get_spinner(which)
   c_template <- template
   c_static <- match.arg(static)
   c_state <- 1L
   c_first <- TRUE
   c_col <- 1L
-  c_width <- console_width()
+  c_width <- 0L
   c_last <- Sys.time() - as.difftime(1, units = "secs")
   c_int <- as.difftime(c_spinner$interval / 1000, units = "secs")
 
@@ -152,17 +203,31 @@ make_spinner <- function(which = NULL, stream = stderr(), template = "{spin}",
     c_first <<- TRUE
     c_col <<- 1L
     c_last <<- Sys.time()
-    if (is_dynamic_tty()) clear_line() else cat("\n", file = c_stream)
+    if (is_dynamic_tty(c_stream)) clear_line() else cat("\n", file = c_stream)
     invisible(c_res)
   }
 
-  if (is_dynamic_tty()) {
+  if (is_dynamic_tty(c_stream)) {
     c_res$spin <- function(template = NULL) {
       if (!is.null(template)) c_template <<- template
       if (throttle()) return()
       line <- sub("{spin}", c_spinner$frames[[c_state]], c_template,
                   fixed = TRUE)
-      cat("\r", line, sep = "", file = stream)
+      line_width <- ansi_nchar(line)
+      if (is_ansi_tty(c_stream)) {
+        cat("\r", line, ANSI_EL, sep = "", file = c_stream)
+      } else {
+        # extra padding in case the line width has changed
+        # so that we don't get any garbage in the output
+        padding <- if (line_width < c_width) {
+          paste0(rep(" ", line_width), collapse = "")
+        } else {
+          ""
+        }
+        cat("\r", line, padding, sep = "", file = c_stream)
+      }
+      # save the new line width
+      c_width <<- line_width
       inc()
     }
 
@@ -174,7 +239,7 @@ make_spinner <- function(which = NULL, stream = stderr(), template = "{spin}",
         if (throttle()) return()
         cat(".", file = c_stream)
         c_col <<- c_col + 1L
-        if (c_col == c_width) {
+        if (c_col == console_width()) {
           cat("\n", file = c_stream)
           c_col <<- 1L
         }
@@ -186,7 +251,7 @@ make_spinner <- function(which = NULL, stream = stderr(), template = "{spin}",
         if (throttle()) return()
         line <- sub("{spin}", c_spinner$frames[[c_state]], c_template,
                     fixed = TRUE)
-        cat(line, file = stream)
+        cat(line, file = c_stream)
         inc()
       }
     } else if (c_static == "print_line") {
@@ -195,7 +260,7 @@ make_spinner <- function(which = NULL, stream = stderr(), template = "{spin}",
         if (throttle()) return()
         line <- sub("{spin}", c_spinner$frames[[c_state]], c_template,
                     fixed = TRUE)
-        cat(line, "\n", sep = "", file = stream)
+        cat(line, "\n", sep = "", file = c_stream)
         inc()
       }
     } else if (c_static == "silent") {
@@ -224,17 +289,23 @@ print.cli_spinner <- function(x, ...) {
 #'
 #' Each spinner is shown for about 2-3 seconds.
 #'
+#' @details
+#'
+#' ```{asciicast demo-spinners}
+#' #| asciicast_at = "all",
+#' #| asciicast_knitr_output = "svg",
+#' #| asciicast_cursor = FALSE,
+#' #| asciicast_end_wait = 0
+#' demo_spinners("clock")
+#' ```
+#'
 #' @param which Character vector, which spinners to demo.
 #'
 #' @family spinners
 #' @export
-#' @examples
-#' \dontrun{
-#'   demo_spinners(sample(list_spinners(), 10))
-#' }
 
 demo_spinners <- function(which = NULL) {
-  assert_that(is.null(which) || is.character(which))
+  stopifnot(is.null(which) || is.character(which))
 
   all <- list_spinners()
   which <- which %||% all
@@ -257,7 +328,7 @@ demo_spinners <- function(which = NULL) {
   }
 }
 
-demo_spinners_terminal <- function() {
+demo_spinners_terminal <- function(ticks = 100 * 3000) {
   up <- function(n) cat(paste0("\u001B[", n, "A"))
   show <- function() cat("\u001b[?25h")
   hide <- function() cat("\u001b[?25l")
@@ -283,7 +354,7 @@ demo_spinners_terminal <- function() {
 
   hide()
 
-  for (tick in 0:1000000) {
+  for (tick in 0:ticks) {
     tic <- Sys.time()
     wframe <- trunc(tick / intervals) %% num_frames + 1
     sp <- mapply(frames, wframe, FUN = "[")
