@@ -1,15 +1,21 @@
 
-#' Update the status bar
+#' Update the status bar (superseded)
+#'
+#' @description
+#' **The `cli_status_*()` functions are superseded by
+#' the [cli_progress_message()] and [cli_progress_step()] functions,
+#' because they have a better default behavior.**
 #'
 #' The status bar is the last line of the terminal. cli apps can use this
 #' to show status information, progress bars, etc. The status bar is kept
 #' intact by all semantic cli output.
 #'
+#' @details
 #' Use [cli_status_clear()] to clear the status bar.
 #'
 #' Often status messages are associated with processes. E.g. the app starts
 #' downloading a large file, so it sets the status bar accordingly. Once the
-#' download is done (or failed), the app typically updates the status bar
+#' download is done (or has failed), the app typically updates the status bar
 #' again. cli automates much of this, via the `msg_done`, `msg_failed`, and
 #' `.auto_result` arguments. See examples below.
 #'
@@ -20,48 +26,62 @@
 #' @param msg_done The message to use when the message is cleared, when
 #'   the calculation finishes successfully. If `.auto_close` is `TRUE`
 #'   and `.auto_result` is `"done"`, then this is printed automatically
-#'   then the calling function (or `.envir`) finishes.
+#'   when the calling function (or `.envir`) finishes.
 #' @param msg_failed The message to use when the message is cleared, when
 #'   the calculation finishes unsuccessfully. If `.auto_close` is `TRUE`
 #'   and `.auto_result` is `"failed"`, then this is printed automatically
-#'   then the calling function (or `.envir`) finishes.
+#'   when the calling function (or `.envir`) finishes.
 #' @param .keep What to do when this status bar is cleared. If `TRUE` then
 #'   the content of this status bar is kept, as regular cli output (the
 #'   screen is scrolled up if needed). If `FALSE`, then this status bar
 #'   is deleted.
 #' @param .auto_close Whether to clear the status bar when the calling
-#'   function finishes (or ‘.envir’ is removed from the stack, if
+#'   function finishes (or `.envir` is removed from the stack, if
 #'   specified).
 #' @param .envir Environment to evaluate the glue expressions in. It is
-#'   also used to auto-clear the status bar if `.auto_close` is `TRUE.
+#'   also used to auto-clear the status bar if `.auto_close` is `TRUE`.
 #' @param .auto_result What to do when auto-closing the status bar.
 #' @return The id of the new status bar container element, invisibly.
 #'
-#' @seealso [cli_process_start] for a higher level interface to the
-#'   status bar, that adds automatic styling.
+#' @seealso Status bars support [inline markup][inline-markup].
+#' @seealso The [cli_progress_message()] and [cli_progress_step()]
+#'   functions, for a superior API.
 #' @family status bar
+#' @family functions supporting inline markup
 #' @export
 
 cli_status <- function(msg, msg_done = paste(msg, "... done"),
                        msg_failed = paste(msg, "... failed"),
                        .keep = FALSE, .auto_close = TRUE,
                        .envir = parent.frame(),
-                       .auto_result = c("clear", "done", "failed")) {
+                       .auto_result = c("clear", "done", "failed", "auto")) {
+
+  id <- new_uuid()
   cli__message(
     "status",
     list(
-      id = NULL,
+      id = id,
       msg = glue_cmd(msg, .envir = .envir),
       msg_done = glue_cmd(msg_done, .envir = .envir),
       msg_failed = glue_cmd(msg_failed, .envir = .envir),
       keep = .keep,
-      auto_result = match.arg(.auto_result)
+      auto_result = match.arg(.auto_result),
+      globalenv = identical(.envir, .GlobalEnv)
     ),
     .auto_close = .auto_close,
     .envir = .envir
   )
+
+  invisible(id)
 }
 
+#' Clear the status bar (superseded)
+#'
+#' @description
+#' **The `cli_status_*()` functions are superseded by
+#' the [cli_progress_message()] and [cli_progress_step()] functions,
+#' because they have a better default behavior.**
+#'
 #' Clear the status bar
 #'
 #' @param id Id of the status bar container to clear. If `id` is not the id
@@ -79,6 +99,8 @@ cli_status <- function(msg, msg_done = paste(msg, "... done"),
 #' @inheritParams cli_status
 #'
 #' @family status bar
+#' @seealso The [cli_progress_message()] and [cli_progress_step()]
+#'   functions, for a superior API.
 #' @export
 
 cli_status_clear <- function(id = NULL, result = c("clear", "done", "failed"),
@@ -89,13 +111,20 @@ cli_status_clear <- function(id = NULL, result = c("clear", "done", "failed"),
     "status_clear",
     list(
       id = id %||% NA_character_,
-      result = match.arg(result),
+      result = match.arg(result[1], c("clear", "done", "failed", "auto")),
       msg_done = if (!is.null(msg_done)) glue_cmd(msg_done, .envir = .envir),
       msg_failed = if (!is.null(msg_failed)) glue_cmd(msg_failed, .envir = .envir)
     )
   )
 }
 
+#' Update the status bar (superseded)
+#'
+#' @description
+#' **The `cli_status_*()` functions are superseded by
+#' the [cli_progress_message()] and [cli_progress_step()] functions,
+#' because they have a better default behavior.**
+#'
 #' Update the status bar
 #'
 #' @param msg Text to update the status bar with. `NULL` if you don't want
@@ -109,7 +138,11 @@ cli_status_clear <- function(id = NULL, result = c("clear", "done", "failed"),
 #' @param .envir Environment to evaluate the glue expressions in.
 #' @return Id of the status bar container.
 #'
+#' @seealso This function supports [inline markup][inline-markup].
+#' @seealso The [cli_progress_message()] and [cli_progress_step()]
+#'   functions, for a superior API.
 #' @family status bar
+#' @family functions supporting inline markup
 #' @export
 
 cli_status_update <- function(id = NULL, msg = NULL, msg_done = NULL,
@@ -126,12 +159,19 @@ cli_status_update <- function(id = NULL, msg = NULL, msg_done = NULL,
 }
 
 #' Indicate the start and termination of some computation in the status bar
+#' (superseded)
+#'
+#' @description
+#' **The `cli_process_*()` functions are superseded by
+#' the [cli_progress_message()] and [cli_progress_step()] functions,
+#' because they have a better default behavior.**
 #'
 #' Typically you call `cli_process_start()` to start the process, and then
 #' `cli_process_done()` when it is done. If an error happens before
 #' `cli_process_done()` is called, then cli automatically shows the message
 #' for unsuccessful termination.
 #'
+#' @details
 #' If you handle the errors of the process or computation, then you can do
 #' the opposite: call `cli_process_start()` with `on_exit = "done"`, and
 #' in the error handler call `cli_process_failed()`. cli will automatically
@@ -157,7 +197,11 @@ cli_status_update <- function(id = NULL, msg = NULL, msg_done = NULL,
 #' @inheritParams cli_status
 #' @return Id of the status bar container.
 #'
+#' @seealso This function supports [inline markup][inline-markup].
+#' @seealso The [cli_progress_message()] and [cli_progress_step()]
+#'   functions, for a superior API.
 #' @family status bar
+#' @family functions supporting inline markup
 #' @export
 #' @examples
 #'
@@ -182,7 +226,7 @@ cli_status_update <- function(id = NULL, msg = NULL, msg_done = NULL,
 
 cli_process_start <- function(msg, msg_done = paste(msg, "... done"),
                               msg_failed = paste(msg, "... failed"),
-                              on_exit = c("failed", "done"),
+                              on_exit = c("auto", "failed", "done"),
                               msg_class = "alert-info",
                               done_class = "alert-success",
                               failed_class = "alert-danger",
@@ -245,7 +289,7 @@ cli_process_failed <- function(id = NULL, msg = NULL, msg_failed = NULL,
 # -----------------------------------------------------------------------
 
 clii_status <- function(app, id, msg, msg_done, msg_failed, keep,
-                        auto_result) {
+                        auto_result, globalenv) {
 
   app$status_bar[[id]] <- list(
     content = "",
@@ -254,6 +298,9 @@ clii_status <- function(app, id, msg, msg_done, msg_failed, keep,
     keep = keep,
     auto_result = auto_result
   )
+  if (isTRUE(getOption("cli.hide_cursor", TRUE)) && !isTRUE(globalenv)) {
+    ansi_hide_cursor(app$output)
+  }
   clii_status_update(app, id, msg, msg_done = NULL, msg_failed = NULL)
 }
 
@@ -262,9 +309,18 @@ clii_status_clear <- function(app, id, result, msg_done, msg_failed) {
   if (is.na(id)) id <- names(app$status_bar)[1]
 
   ## If no active status bar, then ignore
-  if (is.na(id)) return(invisible())
+  if (is.null(id) || is.na(id)) return(invisible())
   if (! id %in% names(app$status_bar)) return(invisible())
 
+  if (result == "auto") {
+    r1 <- random_marker
+    if (identical(returnValue(r1), r1)) {
+      result <- "failed"
+    } else {
+      result <- "done"
+    }
+  }
+
   if (result == "done") {
     msg <- msg_done %||% app$status_bar[[id]]$msg_done
     clii_status_update(app, id, msg, NULL, NULL)
@@ -285,13 +341,16 @@ clii_status_clear <- function(app, id, result, msg_done, msg_failed) {
       ## Not keep? Remove it
       clii__clear_status_bar(app)
     }
+    if (isTRUE(getOption("cli.hide_cursor", TRUE))) {
+      ansi_show_cursor(app$output)
+    }
 
   } else {
     if (app$status_bar[[id]]$keep) {
       ## Keep?
       clii__clear_status_bar(app)
       app$cat(paste0(app$status_bar[[id]]$content, "\n"))
-      app$cat(paste0(app$status_bar[[1]]$content))
+      app$cat(paste0(app$status_bar[[1]]$content, "\r"))
 
     } else {
       ## Not keep? Nothing to output
@@ -302,7 +361,9 @@ clii_status_clear <- function(app, id, result, msg_done, msg_failed) {
   app$status_bar[[id]] <- NULL
 
   ## Switch to the previous one
-  if (length(app$status_bar)) app$cat(paste0(app$status_bar[[1]]$content))
+  if (length(app$status_bar)) {
+    app$cat(paste0(app$status_bar[[1]]$content, "\r"))
+  }
 }
 
 clii_status_update <- function(app, id, msg, msg_done, msg_failed) {
@@ -340,13 +401,18 @@ clii_status_update <- function(app, id, msg, msg_done, msg_failed) {
   ## to the content to make sure we clear up residual content.
   output <- get_real_output(app$output)
   if (is_ansi_tty(output)) {
-    app$cat(paste0("\r", content, ANSI_EL))
+    app$cat(paste0("\r", content, ANSI_EL, "\r"))
   } else if (is_dynamic_tty(output)) {
     nsp <- max(ansi_nchar(current) - ansi_nchar(content), 0)
-    app$cat(paste0("\r", content, strrep(" ", nsp)))
+    app$cat(paste0("\r", content, strrep(" ", nsp), "\r"))
   } else {
     app$cat(paste0(content, "\n"))
   }
+
+  ## Reset timer
+  .Call(clic_tick_reset)
+
+  invisible()
 }
 
 clii__clear_status_bar <- function(app) {
@@ -356,6 +422,6 @@ clii__clear_status_bar <- function(app) {
   } else if (is_dynamic_tty(output)) {
     text <- app$status_bar[[1]]$content
     len <- ansi_nchar(text, type = "width")
-    app$cat(paste0("\r", strrep(" ", len), "\r"))
+    app$cat(paste0("\r", strrep(" ", len + rstudio_r_fix), "\r"))
   }
 }
