@@ -57,3 +57,23 @@ test_that_cli("substitution", {
     })
   }))
 })
+
+test_that("return values are ok when recording (#496)", {
+  expect_snapshot(
+    cli::cli({
+      lid <- cli::cli_ul()
+      cli::cli_li("a bullet")
+      cli::cli_end(lid)
+    })
+  )
+})
+
+test_that("nested cli() (#497)", {
+  expect_snapshot(
+    cli::cli({
+      cli::cli_h1("Header")
+      cli::cli(cli::cli_text("Some text"))
+      cli::cli_text("Some more text")
+    })
+  )
+})
