@@ -1,7 +1,5 @@
 
-context("rules")
-
-test_that("make_line", {
+test_that_cli("make_line", {
 
   expect_equal(make_line(1, "-"), "-")
   expect_equal(make_line(0, "-"), "")
@@ -137,7 +135,7 @@ test_that("line_col", {
   )
 })
 
-test_that("get_line_char", {
+test_that_cli("get_line_char", {
   expect_equal(get_line_char(1), cli::symbol$line)
   expect_equal(get_line_char(2), cli::symbol$double_line)
 
@@ -154,12 +152,7 @@ test_that("get_line_char", {
   expect_equal(get_line_char(c("x", "y", "z")), "xyz")
 })
 
-test_that("print.rule", {
-  withr::with_options(
-    list(cli.width = 20), {
-      out <- capt(rule("foo"))
-      exp <- rebox("── foo ─────────────")
-      expect_equal(out, exp)
-    }
-  )
+test_that("print.cli_rule", {
+  withr::local_options(cli.width = 20)
+  expect_snapshot(rule("foo"))
 })
