@@ -1,8 +1,7 @@
+start_app()
+on.exit(stop_app(), add = TRUE)
 
-context("cli status bar")
-
-setup(start_app())
-teardown(stop_app())
+# We can't easily use snapshot tests here, because they don't capture \r
 
 test_that("create and clear", {
   f <- function() {
@@ -26,9 +25,9 @@ test_that("output while status bar is active", {
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
     "out1\n",
-    "\rstatus1",
-    "\r       \rout2\nstatus1",
-    "\rstatus2",
+    "\rstatus1\r",
+    "\r       \rout2\nstatus1\r",
+    "\rstatus2\r",
     "\r       \r"))
 })
 
@@ -41,7 +40,7 @@ test_that("interpolation", {
   }
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
-    "\rYou see 1+1=2, this is {cli}",
+    "\rYou see 1+1=2, this is {cli}\r",
     "\r                            \r"))
 })
 
@@ -55,8 +54,8 @@ test_that("update", {
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
     "out1\n",
-    "\rstatus1",
-    "\rstatus2",
+    "\rstatus1\r",
+    "\rstatus2\r",
     "\r       \r"))
 })
 
@@ -67,7 +66,7 @@ test_that("keep", {
     cli_status_clear()
   }
   out <- ansi_strip(capt0(f()))
-  expect_equal(out, "\r* This is the current status\n")
+  expect_equal(out, "\r* This is the current status\r\n")
 })
 
 test_that("multiple status bars", {
@@ -82,12 +81,12 @@ test_that("multiple status bars", {
   }
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
-    "\rstatus1",
-    "\r       \rtext1\nstatus1",        # emit text1, restore status1
-    "\rstatus2",                        # show status2
-    "\r       \rtext2\nstatus2",        # emit text2, restore status2
-    "\r       \rstatus1",               # clear status2, restore status1
-    "\r       \rtext3\nstatus1",        # emit text3, restore status1
+    "\rstatus1\r",
+    "\r       \rtext1\nstatus1\r",      # emit text1, restore status1
+    "\rstatus2\r",                      # show status2
+    "\r       \rtext2\nstatus2\r",      # emit text2, restore status2
+    "\r       \rstatus1\r",             # clear status2, restore status1
+    "\r       \rtext3\nstatus1\r",      # emit text3, restore status1
     "\r       \r"))                     # (auto)clear status1
 })
 
@@ -104,7 +103,7 @@ test_that("truncating", {
   }
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
-    "\rEiusmod enim mollit aute aliquip Lore...",
+    "\rEiusmod enim mollit aute aliquip Lore...\r",
     "\r                                        \r"))
 })
 
@@ -147,9 +146,9 @@ test_that("successful termination", {
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
     "out1\n",
-    "\rstatus1",
-    "\r       \rout2\nstatus1",
-    "\rstatus1 ... done\n"
+    "\rstatus1\r",
+    "\r       \rout2\nstatus1\r",
+    "\rstatus1 ... done\r\n"
   ))
 })
 
@@ -164,9 +163,9 @@ test_that("terminate with failed", {
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
     "out1\n",
-    "\rstatus1",
-    "\r       \rout2\nstatus1",
-    "\rstatus1 ... failed\n"
+    "\rstatus1\r",
+    "\r       \rout2\nstatus1\r",
+    "\rstatus1 ... failed\r\n"
   ))
 })
 
@@ -180,9 +179,9 @@ test_that("auto close with success", {
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
     "out1\n",
-    "\rstatus1",
-    "\r       \rout2\nstatus1",
-    "\rstatus1 ... done\n"
+    "\rstatus1\r",
+    "\r       \rout2\nstatus1\r",
+    "\rstatus1 ... done\r\n"
   ))
 })
 
@@ -198,9 +197,9 @@ test_that("auto close wtih failure", {
   out <- ansi_strip(capt0(f()))
   expect_equal(out, paste0(
     "out1\n",
-    "\rstatus1",
-    "\r       \rout2\nstatus1",
-    "\rstatus1 ... failed\n"
+    "\rstatus1\r",
+    "\r       \rout2\nstatus1\r",
+    "\rstatus1 ... failed\r\n"
   ))
 })
 
@@ -218,7 +217,7 @@ test_that("auto close with styling", {
     if (is_interactive()) Sys.sleep(1)
   }
   out <- ansi_strip(capt0(f()))
-  expect_match(out, "status1 ... failed\n")
+  expect_match(out, "status1 ... failed", fixed = TRUE)
 
   f2 <- function() {
     cli_text("out1")
@@ -233,7 +232,7 @@ test_that("auto close with styling", {
     if (is_interactive()) Sys.sleep(1)
   }
   out2 <- ansi_strip(capt0(f2()))
-  expect_match(out2, "status1 ... done\n")
+  expect_match(out2, "status1 ... done", fixed = TRUE)
 })
 
 test_that("process auto close with success", {
@@ -282,16 +281,54 @@ test_that("Emojis are cleaned up properly", {
   exps <- c(
     paste0(
       "out1\n",
-      "\r\U0001F477",
-      "\r  \rout2\n\U0001F477",
-      "\r\u2728",
+      "\r\U0001F477\r",
+      "\r  \rout2\n\U0001F477\r",
+      "\r\u2728\r",
       "\r  \r"),
     paste0(
       "out1\n",
-      "\r<U+0001F477>",
-      "\r  \rout2\n<U+0001F477>",
-      "\r<U+2728>",
+      "\r<U+0001F477>\r",
+      "\r  \rout2\n<U+0001F477>\r",
+      "\r<U+2728>\r",
       "\r  \r")
   )
   expect_true(out %in% exps)
 })
+
+test_that("auto-close with done or failure", {
+  withr::local_options(list(
+    cli.ansi = FALSE,
+    cli.dynamic = TRUE,
+    cli.unicode = FALSE
+  ))
+  f <- function() {
+    cli_text("out1")
+    sb <- cli_process_start("status1")
+    cli_text("out2")
+  }
+  out <- ansi_strip(capt0(f()))
+  expect_match(out, "status1 ... done")
+
+  out <- ansi_strip(capt0(f()))
+  expect_match(out, "status1 ... done")
+
+  # This fails on older R versions, only if f2() is tryCatch()-ed.
+  if (getRversion() < "3.5.0") skip("Needs R 3.5.0")
+
+  f2 <- function() {
+    cli_text("out1")
+    sb <- cli_process_start("status1")
+    cli_text("out2")
+    stop("oops")
+  }
+
+  out2 <- ansi_strip(capt0(tryCatch(f2(), error = function(err) NULL)))
+  expect_match(out2, fixed = TRUE, paste0(
+    "out1\n",
+    "\ri status1\r",
+    "\r         \r",
+    "out2\n",
+    "i status1\r",
+    "\rx status1 ... failed\r\n"
+  ))
+})
